/***************************************************************************
 *
 * gpm-dbus-list.c : Lists names on the SYSTEM and SESSION busses
 *
 * Copyright (C) 2005 Richard Hughes, <richard@hughsie.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 **************************************************************************/

#include <glib.h>
#include <dbus/dbus.h>
#include <dbus/dbus-glib.h>

gboolean
list_names (int dbus_bus_type)
{
	DBusGConnection *connection;
	GError *error;
	DBusGProxy *proxy;
	DBusGPendingCall *call;
	char **name_list;
	int name_list_len;
	int i;

	g_type_init ();

	error = NULL;
	connection = dbus_g_bus_get (dbus_bus_type, &error);
	if (connection == NULL) {
		g_printerr ("Failed to open connection to bus: %s\n",
			    error->message);
		g_error_free (error);
		return 1;
	}

	/* Create a proxy object for the "bus driver" (name "org.freedesktop.DBus") */
	proxy =
	    dbus_g_proxy_new_for_name (connection, "org.freedesktop.DBus",
				       "/org/freedesktop/DBus",
				       "org.freedesktop.DBus");

	/* Call ListNames method */
	call =
	    dbus_g_proxy_begin_call (proxy, "ListNames",
				     DBUS_TYPE_INVALID);

	error = NULL;
	if (!dbus_g_proxy_end_call (proxy, call, &error,
				    DBUS_TYPE_ARRAY, DBUS_TYPE_STRING,
				    &name_list, &name_list_len,
				    DBUS_TYPE_INVALID)) {
		g_printerr ("Failed to complete ListNames call: %s\n",
			    error->message);
		g_error_free (error);
		return 1;
	}

	/* Print the results */
	i = 0;
	while (i < name_list_len) {
		g_assert (name_list[i] != NULL);
		/* only print named services */
		if (name_list[i][0] != ':')
			g_print (" * %s\n", name_list[i]);
		++i;
	}
	g_print ("\n");
	g_assert (name_list[i] == NULL);
	g_strfreev (name_list);
	return 0;
}

int
main (int argc, char **argv)
{
	/* List all services on system bus */
	g_print ("Names on the system message bus:\n");
	list_names (DBUS_BUS_SYSTEM);

	/* List all services on session bus */
	g_print ("Names on the session message bus:\n");
	list_names (DBUS_BUS_SESSION);
	return 0;
}
